from __future__ import annotations

from pathlib import Path


class CondaHookError(Exception):
    pass


class NoCondaExecutableError(CondaHookError):
    def __init__(self):
        super().__init__("failed to find mamba/conda")


class NoEnvFileError(CondaHookError):
    def __init__(self):
        super().__init__("failed to find any env file")


class EnvFileNotFoundError(CondaHookError):
    def __init__(self, path: str | Path):
        super().__init__(f"env file does not exist: {path}")


class NotAFileError(CondaHookError):
    def __init__(self, path: str | Path):
        super().__init__(f"not a file: {path}")


class InvalidEnvFile(CondaHookError):
    def __init__(self, message: str):
        super().__init__(f"invalid env file: {message}")


class EnvDoesNotExistError(CondaHookError):
    def __init__(self, name: str):
        super().__init__(f"environment does not exist: {name}")



from __future__ import annotations

import json
import logging
import subprocess
from pathlib import Path
from typing import Any

import yaml
from yaml import CDumper as Dumper
from yaml import CLoader as Loader

from . import errors, util

LOGGER = logging.getLogger(__name__)
"""A logger to use throughout the module."""

ENV_DEFAULT_PATHS = [
    Path("environment.yml"),
    Path("environment.yaml"),
    Path("conda.yml"),
    Path("conda.yaml"),
]
"""Default names of the Anaconda environment file."""


class EnvironmentFile:
    def __init__(self, path: Path | str | None = None):
        self.path: Path
        self.name: str
        self.content: dict[str, Any] = {}
        self.dependencies: list[str] = []
        self.pip_dependencies: list[str] = []
        self.channels: list[str] = []

        # determine path of env file
        if path is not None:
            path = Path(path)
            if path.exists():
                self.path = path
            else:
                raise errors.NoEnvFileError()
        else:
            for default_path in ENV_DEFAULT_PATHS:
                if default_path.exists():
                    LOGGER.info("automatically found env file: {default_path}")
                    self.path = default_path
                    break
            else:
                raise errors.NoEnvFileError()

        # read env file
        with open(self.path) as fptr:
            self.content = yaml.load(fptr, Loader=Loader)

        # determine env name
        if "name" not in self.content:
            raise errors.InvalidEnvFile("environment name missing")
        self.name = self.content["name"]

        # determine (pip) dependencies
        for dependency in self.content.get("dependencies", []):
            if isinstance(dependency, str):
                self.dependencies.append(dependency)
            elif isinstance(dependency, dict):
                if not isinstance(dependency["pip"], list):
                    raise errors.InvalidEnvFile("pip dependencies should be a list")
                else:
                    for pip_dependency in dependency["pip"]:
                        self.pip_dependencies.append(pip_dependency)

        self.dependencies.sort()
        self.pip_dependencies.sort()

        # read channels
        self.channels = self.content.get("channels", [])

    def write(self, path: Path | None = None):
        if path is None:
            path = self.path

        content: dict[str, Any] = {"name": self.name}
        if self.channels:
            content["channels"] = self.channels
        dependencies: list[str | dict[str, list[str]]] = []

        dependencies += self.dependencies
        if self.pip_dependencies:
            dependencies.append({"pip": self.pip_dependencies})

        content["dependencies"] = dependencies

        with open(path, "w") as fptr:
            yaml.dump(content, fptr, Dumper=Dumper)

    def exists(self) -> bool:
        environments = json.loads(
            subprocess.check_output(
                [util.find_conda_executable(), "env", "list", "--quiet", "--json"]
            )
            .decode()
            .strip()
        )["envs"]
        for environment in environments:
            if Path(environment).name == self.name:
                return True

        return False

    def require_env_exists(self):
        if not self.exists():
            raise errors.EnvDoesNotExistError(self.name)

    def get_installed_dependencies(self) -> list[str]:
        self.require_env_exists()

        exported_environment: dict[str, Any] = yaml.load(
            subprocess.check_output(
                [
                    util.find_conda_executable(),
                    "env",
                    "export",
                    "--from-history",
                    "--quiet",
                    "--name",
                    self.name,
                ],
            ),
            Loader=Loader,
        )

        dependencies = [
            dependency for dependency in exported_environment.get("dependencies", [])
        ]
        dependencies.sort()
        return dependencies

    def update_env(self):
        self.require_env_exists()

        subprocess.run(
            [
                util.find_conda_executable(),
                "env",
                "update",
                "--quiet",
                "--name",
                self.name,
                "--file",
                self.path,
            ],
        )

    def create(self):
        if self.exists():
            LOGGER.warning(f"environment {self.name} exists, do not create")
            return

        cmd = [
            str(util.find_conda_executable()),
            "env",
            "create",
            "--quiet",
            "--name",
            self.name,
            "--file",
            str(self.path),
        ]

        subprocess.check_output(cmd)

    def remove(self):
        if not self.exists():
            LOGGER.warning(f"environment {self.name} does not exists, do not remove")
            return

        subprocess.check_output(
            [
                util.find_conda_executable(),
                "env",
                "remove",
                "--quiet",
                "--name",
                self.name,
            ]
        )



from __future__ import annotations

import logging
import shutil
from pathlib import Path

from . import errors

LOGGER = logging.getLogger(__name__)
"""A logger to use throughout the module."""


def find_conda_executable(
    path: str | Path | None = None, allow_mamba: bool = True
) -> Path:
    """Find mamba/conda executable.

    This routine finds an executable to run conda commands. It prefers mamba since it
    is much faster for many operations. If mamba is not present in the PATH environment
    variable it will try to locate conda instead.

    Returns:
        Path of the mamba/conda executable.

    Raises:
        NoCondaExecutableError: If no mamba/conda executable was found.
    """
    if path is not None:
        path = str(path)

    if allow_mamba:
        result = shutil.which("mamba", path=path)
        if result:
            exe_path = Path(result).resolve()
            LOGGER.info(f"found mamba: {exe_path}")
            return exe_path

    LOGGER.warning("did not find mamba, try to find conda (which might be slower)")

    result = shutil.which("conda", path=path)
    if result:
        exe_path = Path(result).resolve()
        LOGGER.info(f"found conda: {exe_path}")
        return exe_path

    raise errors.NoCondaExecutableError()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['conda_hooks']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.3.1,<6.0.0']

entry_points = \
{'console_scripts': ['conda_env_store = conda_hooks.env_store:main']}

setup_kwargs = {
    'name': 'conda-hooks',
    'version': '0.4.2',
    'description': 'Keep anaconda environment files up to date',
    'long_description': '# conda-hooks\n\nKeep anaconda environment files up to date with installed packages.\nIn contrast to `conda export` it keeps the channel list intact, sorts packages alphabetically and does not purge `pip` dependencies.\nThis can easily be automated using [pre-commit](https://pre-commit.com/) hooks to automatically check for any missing packages before committing.\n\n[![Build Status](https://img.shields.io/github/workflow/status/f-koehler/conda-hooks/build)](https://github.com/f-koehler/conda-hooks/actions)\n[![codecov](https://codecov.io/gh/f-koehler/conda-hooks/branch/main/graph/badge.svg?token=4XHPAHUDOL)](https://codecov.io/gh/f-koehler/conda-hooks)\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/f-koehler/conda-hooks/main.svg)](https://results.pre-commit.ci/latest/github/f-koehler/conda-hooks/main)\n[![PyPI Version](https://img.shields.io/pypi/v/conda-hooks)](https://pypi.org/project/conda-hooks/)\n![License](https://img.shields.io/pypi/l/conda-hooks?color=blue)\n\n## Installation\n\n### As a Python package\n\nThe `conda_hooks` package is installable as a normal python package, for example via pip:\n\n```bash\npip install conda_hooks\n```\n\n### As a `pre-commit` hook\n\nIn your `.pre-commit-config.yaml` file add\n\n```yaml\nrepos:\n  - repo: https://github.com/f-koehler/conda-hooks\n    rev: "0.4.1"\n    hooks:\n      - id: prettier\n```\n\n## Usage\n\n### Command line\n\nRunning `env_store --help` will print information about the available command line options.\nWe can either specify paths to environment file explicitly\n\n```bash\nconda_env_store environment1.yml env2.yaml src/env3.yml\n```\n\nor use globbing patterns like this:\n\n```bash\nconda_env_store -g **/environment.yml -g **/env.yml\n```\n\nOf course we can combine both methods:\n\n```bash\nconda_env_store -g src/env*.yml environment.yml\n```\n\n### As a `pre-commit` hook\n\nWhen using the `pre-commit` hook we can use the same command line arguments, so please refer to the section above.\nAn example using globbing patterns would be:\n\n```yaml\nrepos:\n  - repo: https://github.com/f-koehler/conda-hooks\n    rev: "0.4.1"\n    hooks:\n      - id: prettier\n        args: ["-g", "**/environment.yml"]\n```\n\n## Roadmap\n\nCurrently missing features that should be implemented in the near future:\n\n- handle pinned versions\n',
    'author': 'Fabian Köhler',
    'author_email': 'fabian.koehler@protonmail.ch',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/f-koehler/conda-hooks',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

import argparse
import logging
from pathlib import Path
from typing import Sequence

from .environment import ENV_DEFAULT_PATHS, EnvironmentFile
from .errors import CondaHookError, EnvFileNotFoundError, NoEnvFileError, NotAFileError

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


def get_argument_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description=(
            "Check if one or multiple conda environment files"
            " are up-to-date with the installed packages."
        )
    )
    parser.add_argument(
        "-g",
        "--glob",
        type=str,
        action="append",
        default=[],
        help=(
            "Globbing pattern used to find environment files"
            " (can be specified multiple times)."
        ),
    )
    parser.add_argument(
        "files", type=Path, nargs="*", default=[], help="Paths to environment files."
    )
    return parser


def get_env_files(args: argparse.Namespace) -> list[Path]:
    files: list[Path] = []
    LOGGER.error(args.glob)
    for glob in args.glob:
        files += list(Path.cwd().glob(glob))
    for file in args.files:
        if not file.exists():
            raise EnvFileNotFoundError(file)
        if not file.is_file():
            raise NotAFileError(file)
        files.append(file)

    if not files:
        files = [file for file in ENV_DEFAULT_PATHS if file.exists() and file.is_file()]

    return [file.resolve() for file in files]


def main(argv: Sequence[str] | None = None):

    try:
        parser = get_argument_parser()
        args = parser.parse_args(argv)
        files = get_env_files(args)

        if not files:
            raise NoEnvFileError()

        for file in files:
            env = EnvironmentFile(file)

            if env.exists():
                for dep in env.get_installed_dependencies():
                    if dep not in env.dependencies:
                        env.dependencies.append(dep)

            env.dependencies.sort()
            env.write()
    except CondaHookError as e:
        LOGGER.error(f"conda-hooks error: {e}")


if __name__ == "__main__":
    main()
